package jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

public class PersistenceSample {
	public static void main(String[] args) {

		Address address = new Address("2", "rue Pasteur", "Paris", "75015");
		persistAddress(address);

		Address addressWithId = readAddress(address);

		Person person = new Person("Paul", "Dupont", 25, addressWithId);
		persistPerson(person);

	}

	private static void persistAddress(Address address) {
		try {

			String insertAddressQuery = "INSERT INTO address (number, street, city, postal_code) VALUES (?,?,?,?)";

			Connection dbConnection = JdbcConnection.getConnection();
			PreparedStatement preparedStatement = dbConnection.prepareStatement(insertAddressQuery);

			//Set values of parameters in the query.
			preparedStatement.setString(1, address.getNumber());
			preparedStatement.setString(2, address.getStreet());
			preparedStatement.setString(3, address.getCity());
			preparedStatement.setString(4, address.getPostalCode());

			preparedStatement.executeUpdate();

			preparedStatement.close();
		} catch (SQLException se) {
			System.err.println(se.getMessage());
		}
	}

	private static Address readAddress(Address address) {
		Address readAddress = new Address();
		try {

			String selectAddressQuery = "SELECT * FROM address AS a WHERE a.number = ? AND a.street = ? AND a.city = ?";

			Connection dbConnection = JdbcConnection.getConnection();
			PreparedStatement preparedStatement = dbConnection.prepareStatement(selectAddressQuery);

			preparedStatement.setString(1, address.getNumber());
			preparedStatement.setString(2, address.getStreet());
			preparedStatement.setString(3, address.getCity());

			ResultSet result = preparedStatement.executeQuery();

			while (result.next()) {
				readAddress.setId(result.getString("id"));
				readAddress.setNumber(result.getString("number"));
				readAddress.setStreet(result.getString("street"));
				readAddress.setCity(result.getString("city"));
				readAddress.setPostalCode(result.getString("postal_code"));
			}

			preparedStatement.close();

		} catch (SQLException se) {
			System.err.println(se.getMessage());
		}
		return readAddress;
	}

	private static void persistPerson(Person person) {
		try {

			String insertAddressQuery = "INSERT INTO person (firstname, lastname, age, id_address) VALUES (?,?,?,?)";

			Connection dbConnection = JdbcConnection.getConnection();
			PreparedStatement preparedStatement = dbConnection.prepareStatement(insertAddressQuery);

			preparedStatement.setString(1, person.getFirstname());
			preparedStatement.setString(2, person.getLastname());
			preparedStatement.setInt(3, person.getAge());
			preparedStatement.setString(4, person.getAddress().getId());

			preparedStatement.executeUpdate();
			
			//This is a good way to get the generated primary key value for the inserted row.
			ResultSet keys = preparedStatement.getGeneratedKeys();
			keys.next();
			System.out.println("Generated Key : " + keys.getInt(1));

			preparedStatement.close();
		} catch (SQLException se) {
			System.err.println(se.getMessage());
		}
	}

}
